'use strict';
/*
	File: model.js
	Class: RollTheDice
	
	A contrived simple game -- not guaranteed to be fun!
	But aims to show a little logic and messaging that might
	help you get started on your own more interesting
	game.
	
	Game Rules:
	
	Player vs Computer.
	Each takes turns rolling dice (two).
	First to 30 wins.
	Sum of dice is added to score.
	Except:
	Unlucky pair of ones, decrease score by 10.
	Lucky pair of twos, increase score by 15.
	
	NOTE WELL
	Several ways you can "run" this code.
	1. There is a test case ready to go (see "test" directory)
	2. There is a view ready to go (see "ui" directory)
	3. At the bottom of file there is a small script.
	So if you have a Node.js editor (e.g. Notepad++/other)
	setup, you could just run this file.	
	
	//framework
	1) Code tagged with the tag //framework requires
	a minor edit or no edit (comment by tag will inform)
	2) The expression "extends superClass" below is also
	part of the framework support (no edit required)
*/

//framework - no edit required
let superClass = class {};
try { superClass = Model; } catch(ex) {}

class RollTheDice extends superClass {
	/*ivars 
		playerPoints
		computerPoints
		log -- last action
		activity -- current activity
		derivedValues (for ui):
		message (getMessage)
		tip (getTip)
		
	*/
	
	//------------------------------------------------	
	
	constructor() {
		super();	
		this.playerPoints = 0;
		this.computerPoints = 0;
		this.log = null;		
		this.activity = null;
		//Player always goes first (we could improve this)
		this.setActivityAsWaiting();
	}
	
	//------------------------------------------------	
	//Getters (Basic)
	
	getPlayerPoints() { return this.playerPoints; }
	
	getComputerPoints() { return this.computerPoints; }	
	
	getLog() { return this.log; }	
	
	getActivity() { return this.activity; }		
	
	setActivity(msg) {
		this.activity = msg;
	}	
	
	//------------------------------------------------
	//Getters (derived)
	
	getTip() {
		return 'Click roll or pass (Pair of ones is -10, Pair of twos is +15)';
	}
	
	getMessage() {
		if (this.isTie()) return 'Tied Up';
		const label = this.hasWinner() ? 'Winner' : 'Leader';
		return `${label}: ${this.getLeader()}`;
	}	
	
	isTie() {
		//is it currently tied up
		return this.getPlayerPoints() === this.getComputerPoints();
	}
	
	getLeader() {
		if (this.isTie()) return '';
		if (this.getPlayerPoints() > this.getComputerPoints())
		return 'Player';
		return 'Computer';
	}
	
	getWinningThreshold() {
		return 30;
	}
	
	getWinner() {
		//Return winner (string) or null if no winner yet
		//Assume only one winner
		const
		p = this.getPlayerPoints(),
		c = this.getComputerPoints(),
		threshold = this.getWinningThreshold();
		//Check if no winner
		if (p < threshold && c < threshold) {
			return null;
		}
		//We have winner, leader method will do it for us now
		return this.getLeader();
	}
	
	hasWinner() {
		return this.getWinner() !== null;
	}
	
	isPlayerWinner() {
		return this.hasWinner() && this.getWinner().toLowerCase() == 'player';
	}
	
	isComputerWinner() {
		return this.hasWinner() && this.getWinner().toLowerCase() == 'computer';
	}	
	
	//------------------------------------------------	
	//Public Actions
	
	playerRoll() {
		this.processPlayerRoll(this.rollIt(), this.rollIt());
		this.finishedRoll();
	}
	
	computerRoll() {
		this.processComputerRoll(this.rollIt(), this.rollIt());	
		this.setActivityAsWaiting();
		this.finishedRoll(); 
	}	
	
	finishedRoll() {
		if (this.hasWinner()) {
			this.activity = 'Winner announced';
		}
	}
	
	playerPass() {
		//no operation (do nothing) -- not really needed in 
		//this game; but for a more sophisticated game, an idea.
	}	
	
	setActivityAsWaiting() {
		this.activity = 'Waiting for Player to roll...';	
	}
	
	//------------------------------------------------	
	//Helpers	
	
	processPlayerRoll(roll1, roll2) {
		//This helper method facilitates testing (by moving random to sender)
		//roll1 and roll2 are integers in range 1-6		
		const newScore = this.processRolls('Player', this.playerPoints, roll1, roll2);
		this.playerPoints = newScore;		
	}
	
	processComputerRoll(roll1, roll2) {
		//This helper method facilitates testing (by moving random to sender)
		//roll1 and roll2 are integers in range 1-6			
		const newScore = this.processRolls('Computer', this.computerPoints, roll1, roll2);
		this.computerPoints = newScore;		
	}	
	
	rollIt() {
		//return 1-6 to simulate die roll
		return this.randomBetween(1, 6);
	}
	
	processRolls(label, currentScore, roll1, roll2) {
		//This helper method will also support testing (we can pass the roll values in)
		//Return new score
		//Value 1 through 6 on each cube
		let n1, n2;
		if (this.isUnlucky(roll1, roll2)) {
			this.log = `Sorry ${label}. Bad luck. Pair of ones for -10 points`;
			return currentScore - 10;
		}
		if (this.isLucky(roll1, roll2)) {
			this.log = `Wow ${label}. Very good luck. Pair of twos for 15 points`;
			return currentScore + 15;
		}
	//regular play
	this.log = `${label} rolled a ${roll1} and ${roll2}`;
	return currentScore + roll1 + roll2;
	}	
	
	isUnlucky(n1, n2) {
	return n1 === 1 && n2 === 1;
	}
	
	isLucky(n1, n2) {
	return n1 === 2 && n2 === 2;
	}	
	
	//--------------------------------------------------
	
	toString() {
	return 'Dice Roll Game';
	}		
	
	getTitle() {
	return 'Dice Roll Game';
	}
	
	randomBetween(low, high) {
	//random number between params (inclusive)
	//e.g. between say 1 and 10
	return Math.floor(Math.random() * high) + low;
	}
	
	}
	
	//--------------------------------
	//Playing
	
	function prn(o) { console.log(o); }
	
	const game = new RollTheDice();
	
	game.processComputerRoll(1, 2);
	game.processComputerRoll(6, 3);
	prn(game.getMessage());
	game.processPlayerRoll(2, 2);
	//No winner yet
	prn(game.getMessage());
	
	game.processPlayerRoll(2, 2);
	//Now should have winner
	prn(game.getMessage());
	
	
	
		