"use strict";

/*
	File: controller.js
	Class: Controller
*/

/*
	If using this file as a template for another unit test:
	
		1) There are a few "//framework" tags in this file. A 
		minor edit or no edit is needed (comment by tag will inform)
		2) The expression "extends TestCase" below is also
		part of the framework support (no edit required)
*/

class Controller extends AbstractController {
	
	//----------------------------
	//Virtual - required
	
	newModel() {
		//Override
		//framework -- name of model class
		return new RollTheDice();
	}
	
	//----------------------------
	//Virtual - optional
	
	getRefreshableFields() {
		//framework -- add any field names here you want to auto refresh
		//when controller does "refresh()" action. Model should have
		//getter for each, e.g. getWidth, getHeight, etc.
		return ['playerPoints', 'computerPoints', 'message', 'log', 'tip', 'activity'];
	}		
	
	//----------------------------
	//Attach (connect) element events to handler methods
	
	attachEvents() {
		//Attach field event to event handler
		super.attachEvents();
		
		//actions
		//ui element name, event name, event handler method name
		//the last param (false) means do not auto refresh, we'll do that in handler
		this.attachAction('roll', 'click', 'clickedRoll', false);
		this.attachAction('pass', 'click', 'clickedPass', false);		
	}
		
	//-------------------------------
	//Action event handlers -- actions (e.g. button clicks 'clicked')
	
	clickedRoll() {
		//run player, followed by computer
		this.simulatePlayerRoll(() => this.simulateComputerRoll());
	}	
	
	clickedPass() {
		const game = this.model;
		game.playerPass();
		this.simulateComputerRoll();
	}	
	
	//-----------------------------------------------
	
	simulatePlayerRoll(afterFct) {
		const game = this.model;
		this.disableButtons(true);
		//'#FFCC99' is a color
		this.updateActivity('Player rolling...', '#FFCC99');
		//Delay a few seconds for drama
		//We use a js "setTimeout" to do the delay for us (milliseconds)
		setTimeout(() => this.doPlayerRoll(), 1500);
		//We schedule the "afterFct" when all is done
		setTimeout(afterFct, 1600);		
	}
	
	doPlayerRoll() {
		const game = this.model;
		game.playerRoll();
		this.disableButtons(false);		
		this.refresh();
	}
	
	//-------------------------------	

	simulateComputerRoll() {
		const game = this.model;
		if (game.hasWinner()) return;
		this.disableButtons(true);
		this.updateActivity('Computer rolling...', 'yellow');		
		setTimeout(() => this.doComputerRoll(), 3000);
	}
	
	doComputerRoll() {
		const game = this.model;
		game.computerRoll();
		this.disableButtons(false);		
		this.refresh();
	}
	
	//-------------------------------
	//User friendly type mvc logic here (optional)
				
	refreshState() {
		//framework
		//This gives us an opportunity to enable/disable a button per state, etc.
		//OPTIONAL 
		super.refreshState();
		const win = this.model.hasWinner();

		//If we have winner, game over, disable buttons
		this.disableButtons(win);
		
		//Add a little color (a light green for the win)
		this.elem('message').style.backgroundColor = win ? '#33FF33' : null;
	}	
	
	//-------------------------------	
	//Helpers
	
	disableButtons(disable) {
		//disable buttons if disable=true
		for (let buttonName of ['roll', 'pass']) {
			this.elem(buttonName).disabled = disable;
		}
	}
	
	updateActivity(activityLabel, backColor) {
		this.model.setActivity(activityLabel);	
		this.elem('activity').style.backgroundColor = backColor;		
		this.refreshFields('activity');				
	}		
	
}

//======================================================
//No changes required

function buildController() {
	//one and only controller (global)
	controller = new Controller();
}	


